@@ -5,6 +5,12 @@ from django.db.models import fields
 from south.db import generic
 from south.db.generic import delete_column_constraints, invalidate_table_constraints, copy_column_constraints
 from south.exceptions import ConstraintDropped
+from south.utils.py3 import string_types
+try:
+    from django.utils.encoding import smart_text                    # Django >= 1.5
+except ImportError:
+    from django.utils.encoding import smart_unicode as smart_text   # Django < 1.5
+from django.core.management.color import no_style
 
 class DatabaseOperations(generic.DatabaseOperations):
     """
@@ -40,18 +46,18 @@ class DatabaseOperations(generic.DatabaseOperations):
     def delete_column(self, table_name, name):
         q_table_name, q_name = (self.quote_name(table_name), self.quote_name(name))
 
-        # Zap the indexes
-        for ind in self._find_indexes_for_column(table_name,name):
-            params = {'table_name':q_table_name, 'index_name': ind}
-            sql = self.drop_index_string % params
-            self.execute(sql, [])
-
         # Zap the constraints
         for const in self._find_constraints_for_column(table_name,name):
             params = {'table_name':q_table_name, 'constraint_name': const}
             sql = self.drop_constraint_string % params
             self.execute(sql, [])
 
+        # Zap the indexes
+        for ind in self._find_indexes_for_column(table_name,name):
+            params = {'table_name':q_table_name, 'index_name': ind}
+            sql = self.drop_index_string % params
+            self.execute(sql, [])
+
         # Zap default if exists
         drop_default = self.drop_column_default_sql(table_name, name)
         if drop_default:
@@ -66,16 +72,16 @@ class DatabaseOperations(generic.DatabaseOperations):
 
         sql = """
         SELECT si.name, si.id, sik.colid, sc.name
-        FROM dbo.sysindexes SI WITH (NOLOCK)
-        INNER JOIN dbo.sysindexkeys SIK WITH (NOLOCK)
-            ON  SIK.id = Si.id
-            AND SIK.indid = SI.indid
-        INNER JOIN dbo.syscolumns SC WITH (NOLOCK)
-            ON  SI.id = SC.id
-            AND SIK.colid = SC.colid
-        WHERE SI.indid !=0
-            AND Si.id = OBJECT_ID('%s')
-            AND SC.name = '%s'
+        FROM dbo.sysindexes si WITH (NOLOCK)
+        INNER JOIN dbo.sysindexkeys sik WITH (NOLOCK)
+            ON  sik.id = si.id
+            AND sik.indid = si.indid
+        INNER JOIN dbo.syscolumns sc WITH (NOLOCK)
+            ON  si.id = sc.id
+            AND sik.colid = sc.colid
+        WHERE si.indid !=0
+            AND si.id = OBJECT_ID('%s')
+            AND sc.name = '%s'
         """
         idx = self.execute(sql % (table_name, name), [])
         return [i[0] for i in idx]
@@ -136,7 +142,16 @@ class DatabaseOperations(generic.DatabaseOperations):
             cons_name, type = r[:2]
             if type=='PRIMARY KEY' or type=='UNIQUE':
                 cons = all.setdefault(cons_name, (type,[]))
-                cons[1].append(r[7])
+                sql = '''
+                SELECT COLUMN_NAME
+                FROM INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE RFD
+                WHERE RFD.CONSTRAINT_CATALOG = %s
+                  AND RFD.CONSTRAINT_SCHEMA = %s
+                  AND RFD.TABLE_NAME = %s
+                  AND RFD.CONSTRAINT_NAME = %s
+                '''
+                columns = self.execute(sql, [db_name, schema_name, table_name, cons_name])
+                cons[1].extend(col for col, in columns)
             elif type=='CHECK':
                 cons = (type, r[2])
             elif type=='FOREIGN KEY':
@@ -168,6 +183,11 @@ class DatabaseOperations(generic.DatabaseOperations):
             sch = qn(self._get_schema_name())
             tab = qn(table_name)
             table = ".".join([sch, tab])
+            try:
+                self.delete_foreign_key(table_name, name)
+            except ValueError:
+                # no FK constraint on this field. That's OK.
+                pass
             constraints = self._find_constraints_for_column(table_name, name, False)
             for constraint in constraints.keys():
                 params = dict(table_name = table,
@@ -178,18 +198,12 @@ class DatabaseOperations(generic.DatabaseOperations):
         ret_val = super(DatabaseOperations, self).alter_column(table_name, name, field, explicit_name, ignore_constraints=True)
         
         if not ignore_constraints:
-            unique_field_handled = False
             for cname, (ctype,args) in constraints.items():
                 params = dict(table = table,
                               constraint = qn(cname))
                 if ctype=='UNIQUE':
-                    if len(args)==1:
-                        unique_field_handled = True # 
-                    if len(args)>1 or field.unique:
-                        params['columns'] = ", ".join(map(qn,args))
-                        sql = self.create_unique_sql % params
-                    else:
-                        continue
+                    params['columns'] = ", ".join(map(qn,args))
+                    sql = self.create_unique_sql % params
                 elif ctype=='PRIMARY KEY':
                     params['columns'] = ", ".join(map(qn,args))
                     sql = self.create_primary_key_string % params
@@ -209,9 +223,6 @@ class DatabaseOperations(generic.DatabaseOperations):
                 else:
                     raise NotImplementedError("Don't know how to handle constraints of type "+ type)                    
                 self.execute(sql, [])
-            # Create unique constraint if necessary
-            if field.unique and not unique_field_handled:
-                self.create_unique(table_name, (name,))
             # Create foreign key if necessary
             if field.rel and self.supports_foreign_keys:
                 self.execute(
@@ -222,6 +233,9 @@ class DatabaseOperations(generic.DatabaseOperations):
                         field.rel.to._meta.get_field(field.rel.field_name).column
                     )
                 )
+                model = self.mock_model("FakeModelForIndexCreation", table_name)
+                for stmt in self._get_connection().creation.sql_indexes_for_field(model, field, no_style()):
+                    self.execute(stmt)
 
 
         return ret_val
@@ -246,8 +260,8 @@ class DatabaseOperations(generic.DatabaseOperations):
         conn = self._get_connection()
         value = field.get_db_prep_save(value, connection=conn)
         # This is still a Python object -- nobody expects to need a literal.
-        if isinstance(value, basestring):
-            return smart_unicode(value)
+        if isinstance(value, string_types):
+            return smart_text(value)
         elif isinstance(value, (date,time,datetime)):
             return value.isoformat()
         else:
@@ -288,7 +302,7 @@ class DatabaseOperations(generic.DatabaseOperations):
     # 1) The sql-server-specific call to _fix_field_definition
     # 2) Removing a default, when needed, by calling drop_default and not the more general alter_column
     @invalidate_table_constraints
-    def add_column(self, table_name, name, field, keep_default=True):
+    def add_column(self, table_name, name, field, keep_default=False):
         """
         Adds the column 'name' to the table 'table_name'.
         Uses the 'field' paramater, a django.db.models.fields.Field instance,
@@ -330,7 +344,7 @@ class DatabaseOperations(generic.DatabaseOperations):
             self._fix_field_definition(f)
 
         # Run
-        generic.DatabaseOperations.create_table(self, table_name, field_defs)
+        super(DatabaseOperations, self).create_table(table_name, field_defs)
 
     def _find_referencing_fks(self, table_name):
         "MSSQL does not support cascading FKs when dropping tables, we need to implement."
@@ -402,8 +416,11 @@ class DatabaseOperations(generic.DatabaseOperations):
         params = (self.quote_name(old_table_name), self.quote_name(table_name))
         self.execute('EXEC sp_rename %s, %s' % params)
 
-    _db_type_for_alter_column = generic.alias("_db_positive_type_for_alter_column")
-    _alter_add_column_mods = generic.alias("_alter_add_positive_check")
+    def _db_type_for_alter_column(self, field): 
+        return self._db_positive_type_for_alter_column(DatabaseOperations, field)
+
+    def _alter_add_column_mods(self, field, name, params, sqls):
+        return self._alter_add_positive_check(DatabaseOperations, field, name, params, sqls)
 
     @invalidate_table_constraints
     def delete_foreign_key(self, table_name, column):
@@ -425,6 +442,6 @@ class DatabaseOperations(generic.DatabaseOperations):
         schema = self._get_schema_name()
         indexes = self.execute(find_index_sql, [schema, table_name, column])
         qn = self.quote_name
-        for index in (i[0] for i in indexes):
+        for index in (i[0] for i in indexes if i[0]): # "if i[0]" added because an empty name may return
             self.execute("DROP INDEX %s on %s.%s" % (qn(index), qn(schema), qn(table_name) ))
             
