@@ -1,9 +1,14 @@
-from copy import copy
-from cStringIO import StringIO
+from __future__ import print_function
+
+from copy import copy, deepcopy
 import datetime
 import inspect
 import sys
 import traceback
+try:
+    from cStringIO import StringIO # python 2
+except ImportError:
+    from io import StringIO # python 3
 
 from django.core.management import call_command
 from django.core.management.commands import loaddata
@@ -27,7 +32,7 @@ class Migrator(object):
 
     def print_title(self, target):
         if self.verbosity:
-            print self.title(target)
+            print(self.title(target))
         
     @staticmethod
     def status(target):
@@ -36,7 +41,7 @@ class Migrator(object):
     def print_status(self, migration):
         status = self.status(migration)
         if self.verbosity and status:
-            print status
+            print(status)
 
     @staticmethod
     def orm(migration):
@@ -70,44 +75,46 @@ class Migrator(object):
             '%s\n'
             ' ! The South developers regret this has happened, and would\n'
             ' ! like to gently persuade you to consider a slightly\n'
-            ' ! easier-to-deal-with DBMS.\n'
+            ' ! easier-to-deal-with DBMS (one that supports DDL transactions)\n'
             ' ! NOTE: The error which caused the migration to fail is further up.'
         ) % extra_info
 
-    def run_migration(self, migration):
+    def run_migration(self, migration, database):
         migration_function = self.direction(migration)
         south.db.db.start_transaction()
         try:
             migration_function()
             south.db.db.execute_deferred_sql()
+            if not isinstance(getattr(self, '_wrapper', self), DryRunMigrator):
+                # record us as having done this in the same transaction,
+                # since we're not in a dry run
+                self.record(migration, database)
         except:
             south.db.db.rollback_transaction()
             if not south.db.db.has_ddl_transactions:
-                print self.run_migration_error(migration)
+                print(self.run_migration_error(migration))
+            print("Error in migration: %s" % migration)
             raise
         else:
-            south.db.db.commit_transaction()
+            try:
+                south.db.db.commit_transaction()
+            except:
+                print("Error during commit in migration: %s" % migration)
+                raise
+                
 
-    def run(self, migration):
+    def run(self, migration, database):
         # Get the correct ORM.
         south.db.db.current_orm = self.orm(migration)
-        # If the database doesn't support running DDL inside a transaction
-        # *cough*MySQL*cough* then do a dry run first.
-        if not south.db.db.has_ddl_transactions:
-            dry_run = DryRunMigrator(migrator=self, ignore_fail=False)
-            dry_run.run_migration(migration)
-        return self.run_migration(migration)
+        # If we're not already in a dry run, and the database doesn't support
+        # running DDL inside a transaction, *cough*MySQL*cough* then do a dry
+        # run first.
+        if not isinstance(getattr(self, '_wrapper', self), DryRunMigrator):
+            if not south.db.db.has_ddl_transactions:
+                dry_run = DryRunMigrator(migrator=self, ignore_fail=False)
+                dry_run.run_migration(migration, database)
+        return self.run_migration(migration, database)
 
-    def done_migrate(self, migration, database):
-        south.db.db.start_transaction()
-        try:
-            # Record us as having done this
-            self.record(migration, database)
-        except:
-            south.db.db.rollback_transaction()
-            raise
-        else:
-            south.db.db.commit_transaction()
 
     def send_ran_migration(self, migration):
         ran_migration.send(None,
@@ -122,8 +129,7 @@ class Migrator(object):
         app = migration.migrations._migrations
         migration_name = migration.name()
         self.print_status(migration)
-        result = self.run(migration)
-        self.done_migrate(migration, database)
+        result = self.run(migration, database)
         self.send_ran_migration(migration)
         return result
 
@@ -135,9 +141,10 @@ class MigratorWrapper(object):
     def __init__(self, migrator, *args, **kwargs):
         self._migrator = copy(migrator)
         attributes = dict([(k, getattr(self, k))
-                           for k in self.__class__.__dict__.iterkeys()
+                           for k in self.__class__.__dict__
                            if not k.startswith('__')])
         self._migrator.__dict__.update(attributes)
+        self._migrator.__dict__['_wrapper'] = self
 
     def __getattr__(self, name):
         return getattr(self._migrator, name)
@@ -149,10 +156,13 @@ class DryRunMigrator(MigratorWrapper):
         self._ignore_fail = ignore_fail
 
     def _run_migration(self, migration):
-        if migration.no_dry_run() and self.verbosity:
-            print " - Migration '%s' is marked for no-dry-run." % migration
+        if migration.no_dry_run():
+            if self.verbosity:
+                print(" - Migration '%s' is marked for no-dry-run." % migration)
             return
         south.db.db.dry_run = True
+        # preserve the constraint cache as it can be mutated by the dry run
+        constraint_cache = deepcopy(south.db.db._constraint_cache)
         if self._ignore_fail:
             south.db.db.debug, old_debug = False, south.db.db.debug
         pending_creates = south.db.db.get_pending_creates()
@@ -170,8 +180,11 @@ class DryRunMigrator(MigratorWrapper):
                 south.db.db.debug = old_debug
             south.db.db.clear_run_data(pending_creates)
             south.db.db.dry_run = False
+            # restore the preserved constraint cache from before dry run was
+            # executed
+            south.db.db._constraint_cache = constraint_cache
 
-    def run_migration(self, migration):
+    def run_migration(self, migration, database):
         try:
             self._run_migration(migration)
         except exceptions.FailedDryRun:
@@ -179,17 +192,16 @@ class DryRunMigrator(MigratorWrapper):
                 return False
             raise
 
-    def done_migrate(self, *args, **kwargs):
-        pass
-
     def send_ran_migration(self, *args, **kwargs):
         pass
 
 
 class FakeMigrator(MigratorWrapper):
-    def run(self, migration):
+    def run(self, migration, database):
+        # Don't actually run, just record as if ran
+        self.record(migration, database)
         if self.verbosity:
-            print '   (faked)'
+            print('   (faked)')
 
     def send_ran_migration(self, *args, **kwargs):
         pass
@@ -197,12 +209,12 @@ class FakeMigrator(MigratorWrapper):
 
 class LoadInitialDataMigrator(MigratorWrapper):
     
-    def load_initial_data(self, target):
+    def load_initial_data(self, target, db='default'):
         if target is None or target != target.migrations[-1]:
             return
         # Load initial data, if we ended up at target
         if self.verbosity:
-            print " - Loading initial data for %s." % target.app_label()
+            print(" - Loading initial data for %s." % target.app_label())
         # Override Django's get_apps call temporarily to only load from the
         # current app
         old_get_apps = models.get_apps
@@ -210,7 +222,7 @@ class LoadInitialDataMigrator(MigratorWrapper):
         models.get_apps = new_get_apps
         loaddata.get_apps = new_get_apps
         try:
-            call_command('loaddata', 'initial_data', verbosity=self.verbosity)
+            call_command('loaddata', 'initial_data', verbosity=self.verbosity, database=db)
         finally:
             models.get_apps = old_get_apps
             loaddata.get_apps = old_get_apps
@@ -219,7 +231,7 @@ class LoadInitialDataMigrator(MigratorWrapper):
         migrator = self._migrator
         result = migrator.__class__.migrate_many(migrator, target, migrations, database)
         if result:
-            self.load_initial_data(target)
+            self.load_initial_data(target, db=database)
         return True
 
 
@@ -253,7 +265,11 @@ class Forwards(Migrator):
     def record(migration, database):
         # Record us as having done this
         record = MigrationHistory.for_migration(migration, database)
-        record.applied = datetime.datetime.utcnow()
+        try:
+            from django.utils.timezone import now
+            record.applied = now()
+        except ImportError:
+            record.applied = datetime.datetime.utcnow()
         if database != DEFAULT_DB_ALIAS:
             record.save(using=database)
         else:
